indexing

	description: "Class that covers Posix semaphore routines."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #2 $"

class

	POSIX_UNNAMED_SEMAPHORE


inherit

	POSIX_SEMAPHORE
		export
			{ANY} SEM_VALUE_MAX
		end

	MEMORY
		redefine
			dispose
		end


creation

	create_shared,
	create_unshared


feature {NONE} -- Initialization

	create_shared (a_value: INTEGER) is
			-- creates an unnamed semaphore
		require
			uninitialized: not is_initialized
			semaphores_supported: supports_semaphores
			valid_value: a_value >= 0 and a_value <= SEM_VALUE_MAX
		do
			make_sem_ptr
			shared := True
			safe_call (posix_sem_init (sem, shared, a_value))
		end

	create_unshared (a_value: INTEGER) is
			-- creates an unnamed semaphore
		require
			uninitialized: not is_initialized
			semaphores_supported: supports_semaphores
			valid_value: a_value >= 0 and a_value <= SEM_VALUE_MAX
		do
			make_sem_ptr
			shared := False
			safe_call (posix_sem_init (sem, shared, a_value))
		end


feature {NONE} -- Sempahore creation

	make_sem_ptr is
			-- allocate `sem_ptr', set `sem'
		do
			if sem_ptr = Void then
				create sem_ptr.allocate_and_clear (posix_sem_t_size)
			end
			sem := sem_ptr.ptr
		end


feature -- Commands

	destroy is
			-- destroy the semaphore
		require
			initialized: is_initialized
		do
			safe_call (posix_sem_destroy (sem))
			sem := default_pointer
		ensure
			uninitialized: not is_initialized
		end


feature -- Garbage collection

	dispose is
		local
			r: INTEGER
		do
			if sem /= default_pointer then
				r := posix_sem_destroy (sem)
			end
		end


feature -- Queries

	shared: BOOLEAN


feature {NONE} -- Implementation

	sem_ptr: STDC_BUFFER


invariant

	sem_ptr_valid: sem_ptr /= Void and then sem_ptr.capacity = posix_sem_t_size


end
